/* bztclDecls.h --
   
   Part of: BZTCL
   Contents: declarations for the stubs
   Date: Fri Aug 15, 2003
   
   Abstract
   
	Provides declarations.  This file is included  in both "bztcl.h"
	and "bztclInt.h".
   
   Copyright (c) 2003, 2004 Marco Maggi
   
   The author hereby grant  permission to use, copy, modify, distribute,
   and  license this  software and  its documentation  for  any purpose,
   provided that  existing copyright notices are retained  in all copies
   and that  this notice is  included verbatim in any  distributions. No
   written agreement, license, or royalty fee is required for any of the
   authorized uses.   Modifications to this software  may be copyrighted
   by their  authors and need  not follow the licensing  terms described
   here, provided that the new  terms are clearly indicated on the first
   page of each file where they apply.
   
   IN NO EVENT  SHALL THE AUTHOR OR DISTRIBUTORS BE  LIABLE TO ANY PARTY
   FOR DIRECT,  INDIRECT, SPECIAL, INCIDENTAL,  OR CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE USE OF  THIS SOFTWARE, ITS  DOCUMENTATION, OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR HAVE  BEEN ADVISED  OF THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR AND  DISTRIBUTORS SPECIFICALLY  DISCLAIM  ANY WARRANTIES,
   INCLUDING,   BUT  NOT   LIMITED   TO,  THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A   PARTICULAR    PURPOSE,   AND
   NON-INFRINGEMENT.  THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, AND
   THE   AUTHOR  AND   DISTRIBUTORS  HAVE   NO  OBLIGATION   TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#ifndef __BZTCLDECLS_H
#define __BZTCLDECLS_H 1


/** ------------------------------------------------------------
 ** Typedefs.
 ** ----------------------------------------------------------*/

typedef ClientData			Bztcl_File;
typedef ClientData			Bztcl_Stream;

typedef struct Bztcl_Config {
  int	blockSize;
  int	workFactor;
  int	small;
} Bztcl_Config;

#define BZTCL_DEFAULT_CONFIG		{ 3, 30, 0 }


/** ------------------------------------------------------------
 ** Beginning of prototypes declaration.
 ** ----------------------------------------------------------*/

#ifndef USE_BZTCL_STUB


/** ------------------------------------------------------------
 ** Object compression and decompression functions.
 ** ----------------------------------------------------------*/

EXTERN More_Error Bztcl_CompressObj	_ANSI_ARGS_((Tcl_Obj *srcObj,
						     Bztcl_Config *config,
						     Tcl_Obj **dstObjVar));

EXTERN More_Error Bztcl_UncompressObj	_ANSI_ARGS_((Tcl_Obj *srcObj,
						     Bztcl_Config *config,
						     int size,
						     Tcl_Obj **dstObjVar));


/** ------------------------------------------------------------
 ** Interface to .bz2 files.
 ** ----------------------------------------------------------*/

EXTERN More_Error Bztcl_Open		_ANSI_ARGS_((CONST char *fileName,
						     int openMode,
						     Bztcl_File *tokenVar));

EXTERN More_Error Bztcl_Close		_ANSI_ARGS_((Bztcl_File token));

EXTERN More_Error Bztcl_ReadObj		_ANSI_ARGS_((Bztcl_File token,
						     int number,
						     Tcl_Obj **objVar));

EXTERN More_Error Bztcl_Read		_ANSI_ARGS_((Bztcl_File token,
						     More_Block *blockVar));

EXTERN More_Error Bztcl_WriteObj	_ANSI_ARGS_((Bztcl_File token,
						     Tcl_Obj *objPtr));

EXTERN More_Error Bztcl_Write		_ANSI_ARGS_((Bztcl_File token,
						     More_Block block));

EXTERN void Bztcl_SetSmall	_ANSI_ARGS_((Bztcl_File token, int small));
EXTERN void Bztcl_SetWorkFactor	_ANSI_ARGS_((Bztcl_File token, int workFactor));
EXTERN void Bztcl_SetBlockSize	_ANSI_ARGS_((Bztcl_File token, int blockSize));

EXTERN int Bztcl_Eof		_ANSI_ARGS_((Bztcl_File token));
EXTERN int Bztcl_GetHandle	_ANSI_ARGS_((Bztcl_File token));
EXTERN int Bztcl_GetWorkFactor	_ANSI_ARGS_((Bztcl_File token));
EXTERN int Bztcl_GetBlockSize	_ANSI_ARGS_((Bztcl_File token));
EXTERN int Bztcl_GetSmall	_ANSI_ARGS_((Bztcl_File token));

EXTERN int Bztcl_ReadableFile	_ANSI_ARGS_((Bztcl_File token));
EXTERN int Bztcl_WritableFile	_ANSI_ARGS_((Bztcl_File token));


/** ------------------------------------------------------------
 ** Stream compression and decompression.
 ** ----------------------------------------------------------*/

#define Bztcl_StreamInitCompress(tokenVar, config)	\
			Bztcl_StreamInit((tokenVar), 1, (config))
#define Bztcl_StreamInitDecompress(tokenVar)		\
			Bztcl_StreamInit((tokenVar), 0, (config))

EXTERN More_Error Bztcl_StreamInit	_ANSI_ARGS_((Bztcl_Stream *tokenVar,
						     int compress,
						     Bztcl_Config *config));
EXTERN void       Bztcl_StreamFinal	_ANSI_ARGS_((Bztcl_Stream token));

EXTERN More_Block Bztcl_StreamOutput	_ANSI_ARGS_((Bztcl_Stream token));
EXTERN void	  Bztcl_StreamRead	_ANSI_ARGS_((Bztcl_Stream token,
						     int number));
EXTERN More_Error Bztcl_StreamWrite	_ANSI_ARGS_((Bztcl_Stream token,
						     More_Block *blockPtr));
EXTERN More_Error Bztcl_StreamFlush	_ANSI_ARGS_((Bztcl_Stream token));
EXTERN More_Error Bztcl_StreamFinish	_ANSI_ARGS_((Bztcl_Stream token));

EXTERN void Bztcl_StreamSetBufferSize	_ANSI_ARGS_((Bztcl_Stream token,
						     int size));
EXTERN void Bztcl_StreamSetBufferHysteresis _ANSI_ARGS_((Bztcl_Stream token,
							int hysteresis));
EXTERN int Bztcl_StreamGetBufferSize	_ANSI_ARGS_((Bztcl_Stream token));
EXTERN int Bztcl_StreamGetBufferHysteresis _ANSI_ARGS_((Bztcl_Stream token));

EXTERN int Bztcl_StreamGetWorkFactor	_ANSI_ARGS_((Bztcl_Stream token));
EXTERN int Bztcl_StreamGetBlockSize	_ANSI_ARGS_((Bztcl_Stream token));
EXTERN int Bztcl_StreamGetSmall		_ANSI_ARGS_((Bztcl_Stream token));


/** ------------------------------------------------------------
 ** Object value extractors.
 ** ----------------------------------------------------------*/

EXTERN int Bztcl_GetBlockSizeFromObj	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     int *blockSizeVar));

EXTERN int Bztcl_GetWorkFactorFromObj	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     int *workFactorVar));

/** ------------------------------------------------------------
 ** Channel interface.
 ** ----------------------------------------------------------*/

EXTERN Tcl_Channel Bztcl_MakeChannel	_ANSI_ARGS_((Bztcl_File token));
EXTERN Tcl_Channel Bztcl_MakeTransform	_ANSI_ARGS_((Bztcl_Stream input,
						     Bztcl_Stream output,
						     Tcl_Channel subChannel));


/** ------------------------------------------------------------
 ** Miscellaneous functions.
 ** ----------------------------------------------------------*/

EXTERN int		Bztcl_IsWorkFactor	_ANSI_ARGS_((int workFactor));
EXTERN int		Bztcl_IsBlockSize	_ANSI_ARGS_((int blockSize));
EXTERN CONST char *	Bztcl_Version		_ANSI_ARGS_((void));


/** ------------------------------------------------------------
 ** End of prototypes declaration.
 ** ----------------------------------------------------------*/

#endif /* not defined USE_BZTCL_STUB */


/** ------------------------------------------------------------
 ** Stubs table.
 ** ----------------------------------------------------------*/

typedef struct Bztcl_StubTable {
  
  /* Compress functions. */

  More_Error (*bztcl_CompressObj)	_ANSI_ARGS_((Tcl_Obj *srcObj,
						     Bztcl_Config *cfg,
						     Tcl_Obj **dstObjVar));

  More_Error (*bztcl_UncompressObj)	_ANSI_ARGS_((Tcl_Obj *srcObj,
						     Bztcl_Config *cfg,
						     int size,
						     Tcl_Obj **dstObjVar));

  /* File interface. */

  More_Error (*bztcl_Open)		_ANSI_ARGS_((CONST char *fileName,
						     int openMode,
						     Bztcl_File *tokenVar));

  More_Error (*bztcl_Close)		_ANSI_ARGS_((Bztcl_File token));

  More_Error (*bztcl_ReadObj)		_ANSI_ARGS_((Bztcl_File token,
						     int number,
						     Tcl_Obj **objVar));

  More_Error (*bztcl_Read)		_ANSI_ARGS_((Bztcl_File token,
						     More_Block *blockVar));

  More_Error (*bztcl_WriteObj)		_ANSI_ARGS_((Bztcl_File token,
						     Tcl_Obj *objPtr));

  More_Error (*bztcl_Write)		_ANSI_ARGS_((Bztcl_File token,
						     More_Block block));

  void (*bztcl_SetSmall)	_ANSI_ARGS_((Bztcl_File token, int small));
  void (*bztcl_SetWorkFactor)	_ANSI_ARGS_((Bztcl_File token, int workFactor));
  void (*bztcl_SetBlockSize)	_ANSI_ARGS_((Bztcl_File token, int blockSize));

  int (*bztcl_Eof)		_ANSI_ARGS_((Bztcl_File token));
  int (*bztcl_GetHandle)	_ANSI_ARGS_((Bztcl_File token));
  int (*bztcl_GetWorkFactor)	_ANSI_ARGS_((Bztcl_File token));
  int (*bztcl_GetBlockSize)	_ANSI_ARGS_((Bztcl_File token));
  int (*bztcl_GetSmall)		_ANSI_ARGS_((Bztcl_File token));

  int (*bztcl_ReadableFile)	_ANSI_ARGS_((Bztcl_File token));
  int (*bztcl_WritableFile)	_ANSI_ARGS_((Bztcl_File token));

  /* Stream interface. */

  More_Error (*bztcl_StreamInit)	_ANSI_ARGS_((Bztcl_Stream *tokenVar,
						     int compression,
						     Bztcl_Config *config));
  void       (*bztcl_StreamFinal)	_ANSI_ARGS_((Bztcl_Stream token));

  More_Block (*bztcl_StreamOutput)	_ANSI_ARGS_((Bztcl_Stream token));
  void	     (*bztcl_StreamRead)	_ANSI_ARGS_((Bztcl_Stream token,
						     int number));
  More_Error (*bztcl_StreamWrite)	_ANSI_ARGS_((Bztcl_Stream token,
						     More_Block *blockPtr));
  More_Error (*bztcl_StreamFlush)	_ANSI_ARGS_((Bztcl_Stream token));
  More_Error (*bztcl_StreamFinish)	_ANSI_ARGS_((Bztcl_Stream token));

  void	(*bztcl_StreamSetBufferSize)	_ANSI_ARGS_((Bztcl_Stream token,
						     int size));
  void	(*bztcl_StreamSetBufferHysteresis) _ANSI_ARGS_((Bztcl_Stream token,
							int hysteresis));
  int	(*bztcl_StreamGetBufferSize)	_ANSI_ARGS_((Bztcl_Stream token));
  int	(*bztcl_StreamGetBufferHysteresis) _ANSI_ARGS_((Bztcl_Stream token));

  int	(*bztcl_StreamGetWorkFactor)	_ANSI_ARGS_((Bztcl_Stream token));
  int	(*bztcl_StreamGetBlockSize)	_ANSI_ARGS_((Bztcl_Stream token));
  int	(*bztcl_StreamGetSmall)		_ANSI_ARGS_((Bztcl_Stream token));

  /* Value extractors. */

  int (*bztcl_GetBlockSizeFromObj)	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     int *blockSizeVar));

  int (*bztcl_GetWorkFactorFromObj)	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     int *workFactorVar));
  /* Channel interface. */

  Tcl_Channel (*bztcl_MakeChannel)	_ANSI_ARGS_((Bztcl_File token));
  Tcl_Channel (*bztcl_MakeTransform)	_ANSI_ARGS_((Bztcl_Stream input,
						     Bztcl_Stream output,
						     Tcl_Channel subChannel));

  /* Miscellaneous functions. */

  int		(*bztcl_IsWorkFactor)	_ANSI_ARGS_((int workFactor));
  int		(*bztcl_IsBlockSize)	_ANSI_ARGS_((int blockSize));
  CONST char *	(*bztcl_Version)	_ANSI_ARGS_((void));

} Bztcl_StubTable;


/** ------------------------------------------------------------
 ** Stubs declarations.
 ** ----------------------------------------------------------*/

#ifdef USE_BZTCL_STUB

EXTERN Bztcl_StubTable *	bztclStubPtr;

EXTERN CONST char *	Bztcl_InitStub	_ANSI_ARGS_((Tcl_Interp *interp,
						     char *version,
						     int exact));

/* Compression/decompression. */

#define Bztcl_CompressObj		(bztclStubPtr)->bztcl_CompressObj
#define Bztcl_UncompressObj		(bztclStubPtr)->bztcl_UncompressObj

/* File interface. */

#define Bztcl_Open			(bztclStubPtr)->bztcl_Open
#define Bztcl_Close			(bztclStubPtr)->bztcl_Close
#define Bztcl_ReadObj			(bztclStubPtr)->bztcl_ReadObj
#define Bztcl_Read			(bztclStubPtr)->bztcl_Read
#define Bztcl_WriteObj			(bztclStubPtr)->bztcl_WriteObj
#define Bztcl_Write			(bztclStubPtr)->bztcl_Write

#define Bztcl_SetSmall			(bztclStubPtr)->bztcl_SetSmall
#define Bztcl_SetWorkFactor		(bztclStubPtr)->bztcl_SetWorkFactor
#define Bztcl_SetBlockSize		(bztclStubPtr)->bztcl_SetBlockSize

#define Bztcl_Eof			(bztclStubPtr)->bztcl_Eof

#define Bztcl_GetHandle			(bztclStubPtr)->bztcl_GetHandle
#define Bztcl_GetWorkFactor		(bztclStubPtr)->bztcl_GetWorkFactor
#define Bztcl_GetBlockSize		(bztclStubPtr)->bztcl_GetBlockSize
#define Bztcl_GetSmall			(bztclStubPtr)->bztcl_GetSmall

#define Bztcl_ReadableFile		(bztclStubPtr)->bztcl_ReadableFile
#define Bztcl_WritableFile		(bztclStubPtr)->bztcl_WritableFile

/* Stream interface. */

#define Bztcl_StreamInit		(bztclStubPtr)->bztcl_StreamInit
#define Bztcl_StreamFinal		(bztclStubPtr)->bztcl_StreamFinal
#define Bztcl_StreamOutput		(bztclStubPtr)->bztcl_StreamOutput
#define Bztcl_StreamRead		(bztclStubPtr)->bztcl_StreamRead
#define Bztcl_StreamWrite		(bztclStubPtr)->bztcl_StreamWrite
#define Bztcl_StreamFlush		(bztclStubPtr)->bztcl_StreamFlush
#define Bztcl_StreamFinish		(bztclStubPtr)->bztcl_StreamFinish
#define Bztcl_StreamConfigOutbuf	(bztclStubPtr)->bztcl_StreamConfigOutbuf

#define Bztcl_StreamSetBufferSize	(bztclStubPtr)->bztcl_StreamSetBufferSize
#define Bztcl_StreamSetBufferHysteresis	(bztclStubPtr)->bztcl_StreamSetBufferHysteresis
#define Bztcl_StreamGetBufferSize	(bztclStubPtr)->bztcl_StreamGetBufferSize
#define Bztcl_StreamGetBufferHysteresis	(bztclStubPtr)->bztcl_StreamGetBufferHysteresis

#define Bztcl_StreamGetBlockSize	(bztclStubPtr)->bztcl_StreamGetBlockSize
#define Bztcl_StreamGetWorkFactor	(bztclStubPtr)->bztcl_StreamGetWorkFactor
#define Bztcl_StreamGetSmall		(bztclStubPtr)->bztcl_StreamGetSmall

/* Value extractors. */

#define Bztcl_GetBlockSizeFromObj	(bztclStubPtr)->bztcl_GetBlockSizeFromObj
#define Bztcl_GetWorkFactorFromObj	(bztclStubPtr)->bztcl_GetWorkFactorFromObj

/* Channel interface. */

#define Bztcl_MakeChannel		(bztclStubPtr)->bztcl_MakeChannel
#define Bztcl_MakeTransform		(bztclStubPtr)->bztcl_MakeTransform

/* Miscellaneous functions. */

#define Bztcl_IsWorkFactor		(bztclStubPtr)->bztcl_IsWorkFactor
#define Bztcl_IsBlockSize		(bztclStubPtr)->bztcl_IsBlockSize
#define Bztcl_Version			(bztclStubPtr)->bztcl_Version

#endif /* defined USE_BZTCL_STUB */
#endif /* __BZTCLDECLS_H */

/* end of file */
