/* init.c --
   
   Part of: BZTCL
   Contents: initialisation functions
   Date: Tue May  7, 2002
   
   Abstract
   
	This file defines the initialisation functions.   
   
   Copyright (c) 2002, 2003, 2004 Marco Maggi
   
   The author hereby grant  permission to use, copy, modify, distribute,
   and  license this  software and  its documentation  for  any purpose,
   provided that  existing copyright notices are retained  in all copies
   and that  this notice is  included verbatim in any  distributions. No
   written agreement, license, or royalty fee is required for any of the
   authorized uses.   Modifications to this software  may be copyrighted
   by their  authors and need  not follow the licensing  terms described
   here, provided that the new  terms are clearly indicated on the first
   page of each file where they apply.
   
   IN NO EVENT  SHALL THE AUTHOR OR DISTRIBUTORS BE  LIABLE TO ANY PARTY
   FOR DIRECT,  INDIRECT, SPECIAL, INCIDENTAL,  OR CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE USE OF  THIS SOFTWARE, ITS  DOCUMENTATION, OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR HAVE  BEEN ADVISED  OF THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR AND  DISTRIBUTORS SPECIFICALLY  DISCLAIM  ANY WARRANTIES,
   INCLUDING,   BUT  NOT   LIMITED   TO,  THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A   PARTICULAR    PURPOSE,   AND
   NON-INFRINGEMENT.  THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, AND
   THE   AUTHOR  AND   DISTRIBUTORS  HAVE   NO  OBLIGATION   TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#include "bztclInt.h"


/* Bztcl_Init --

	Initialises the package.

   Arguments:

	interp -	The interpreter in which create
			the commands.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

EXTERN int
Bztcl_Init (Tcl_Interp *interp)
{
  int			e;


#ifdef USE_TCL_STUBS
  if (Tcl_InitStubs(interp, "8.4", 0) == NULL)
    {
      return TCL_ERROR;
    }
#endif

#ifdef USE_TCLMORE_STUB
  if (More_InitStub(interp, TCLMORE_QUOTED_VERSION, 0) == NULL)
    {
      return TCL_ERROR;
    }
#endif

  e = Tcl_PkgProvideEx(interp,
		       TCL_QUOTED_PACKAGE_NAME,
		       TCL_QUOTED_PACKAGE_VERSION,
		       (ClientData) &bztclStub);
  if (e == TCL_OK)
    {
      e = Bztcl_CreateCommands(interp);
    }
  return e;
}

/* Bztcl_SafeInit --

	Initialises the package in a safe interpreter.

   Arguments:

	interp -	The interpreter in which create
			the commands.

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

EXTERN int
Bztcl_SafeInit (Tcl_Interp *interp)
{
  return Bztcl_Init(interp);
}

/* end of file */
