m4_divert(-1)m4_dnl
# autoconf.m4 --
# 
# Part of: DevelStuff
# Contents: additional macros for "configure"
# Date: Mon Dec 30, 2002
# 
# Abstract
# 
#	This is a collection of GNU m4 macros that can be used to
#	preprocess a "configure.in" file. These macros are additional
#	to the GNU Autoconf macros.
# 
# Copyright (c)  2002, 2003, 2004  Marco Maggi
# 
# This file  is free software you  can redistribute it  and/or modify it
# under the terms of the GNU  General Public License as published by the
# Free Software  Foundation; either version  2, or (at your  option) any
# later version.
# 
# This  file is  distributed in  the hope  that it  will be  useful, but
# WITHOUT   ANY  WARRANTY;   without  even   the  implied   warranty  of
# MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
# General Public License for more details.
# 
# You  should have received  a copy  of the  GNU General  Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation,  Inc., 59  Temple Place -  Suite 330,  Boston, MA
# 02111-1307, USA.
# 

#PAGE
## ------------------------------------------------------------
## Meta macros.
## ------------------------------------------------------------

m4_changequote({, })

m4_define({ds_error},
{m4_errprint(develstuff *** error:m4___file__:m4___line__: $1
)})

m4_define({ds_warning},
{m4_errprint(develstuff *** warning:m4___file__:m4___line__: $1
)})

m4_define({ds_message},
{m4_errprint(develstuff:m4___file__:m4___line__: $1
)})

m4_define({ds_provide}, {m4_define({$1_invoked})})

m4_define({ds_require},
{m4_ifdef({$1_invoked},,{ds_error({prerequisite violation: $1})})
})

m4_define({ds_defun}, {m4_define({$1}, {ds_provide({$1})$2})})

m4_define({ds_bash}, {m4_esyscmd({bash -c "$1"})})

m4_define({ds_isfile},
{m4_ifelse(ds_bash({if test -f "$1"; then echo yes; else echo no; fi}), {yes
}, {$2}, {$3})})

m4_define({ds_isdir},
{m4_ifelse(ds_bash({if test -d "$1"; then echo yes; else echo no; fi}), {yes
}, {$2}, {$3})})

m4_define({ds_comment_section}, {#PAGE
## ------------------------------------------------------------
## $1
## ------------------------------------------------------------})

#PAGE
## ------------------------------------------------------------
## Setup macros.
## ------------------------------------------------------------

ds_defun({ds_package_name}, {m4_divert(-1)m4_dnl
m4_define({ds_global_package_name}, {$1})m4_dnl
m4_define({ds_global_package_uppercase}, {m4_translit({$1}, {a-z}, {A-Z})})
m4_define({ds_global_package_lowercase}, {m4_translit({$1}, {A-Z}, {a-z})})
m4_divert(0)m4_dnl})

ds_defun({ds_package_version}, {m4_divert(-1)m4_dnl
m4_define({ds_global_package_version_major}, {$1})
m4_define({ds_global_package_version_minor}, {$2})
m4_define({ds_global_package_version_patch}, {$3})

m4_ifelse({$4}, {alpha},  {m4_define({ds_global_package_version_state}, {a})})
m4_ifelse({$4}, {beta},   {m4_define({ds_global_package_version_state}, {b})})
m4_ifelse({$4}, {stable}, {m4_define({ds_global_package_version_state}, {.})})

m4_define({ds_global_package_version}, {$1.$2.$3})
m4_define({ds_global_package_nodot_version}, {$1$2})
m4_define({ds_global_package_xversion}, {$1.$2{}ds_global_package_version_state{}$3})
m4_divert(0)m4_dnl})

ds_defun({ds_author_name}, {m4_define({ds_global_author_name}, {$1})m4_dnl})

ds_defun({ds_author_email}, {m4_define({ds_global_author_email}, {$1})m4_dnl})

ds_defun({ds_copyright_years},
{m4_define({ds_global_copyright_years}, {$1})m4_dnl})

ds_defun({ds_config_dir}, {m4_define({ds_global_config_dir}, {$1})m4_dnl})

ds_defun({ds_source_file}, {m4_define({ds_global_source_file}, {$1})m4_dnl})

#PAGE
## ------------------------------------------------------------
## Initialisation macros.
## ------------------------------------------------------------

ds_defun({ds_init}, {ds_comment_section({Initialisation.})
m4_divert(-1)m4_dnl
ds_require({ds_package_name})
ds_require({ds_package_version})
ds_require({ds_author_name})
ds_require({ds_author_email})
ds_require({ds_copyright_years})
m4_divert(0)m4_dnl

AC_INIT([ds_global_package_name], [ds_global_package_version], [ds_global_author_email])
AC_PREREQ(2.57)
AC_COPYRIGHT([Copyright (C) ds_global_copyright_years by ds_global_author_name])
AC_REVISION([$Revision: 1.1.1.131 $])

m4_ifdef({ds_global_source_file}, {AC_CONFIG_SRCDIR([ds_global_source_file])})m4_dnl
ds_select_config_dir

AC_SUBST([PACKAGE_XVERSION], [ds_global_package_xversion])
AC_DEFINE([PACKAGE_XVERSION], ["ds_global_package_xversion"], [full version])
AC_SUBST([PACKAGE_NODOT_VERSION], [ds_global_package_nodot_version])
AC_SUBST([PACKAGE_NAME_UPPERCASE], [ds_global_package_uppercase])
AC_SUBST([PACKAGE_NAME_LOWERCASE], [ds_global_package_lowercase])

AC_SUBST([PACKAGE_VERSION_MAJOR], [ds_global_package_version_major])
AC_SUBST([PACKAGE_VERSION_MINOR], [ds_global_package_version_minor])
AC_SUBST([PACKAGE_VERSION_PATCH], [ds_global_package_version_patch])

ds_common_programs
})


ds_defun({ds_select_config_dir},
{m4_ifdef({ds_global_config_dir}, {ds_init_config_dir({ds_global_config_dir})}, {
ds_isdir({tclconfig}, {ds_init_config_dir({tclconfig})})m4_dnl
ds_isdir({gnuconfig}, {ds_init_config_dir({gnuconfig})})m4_dnl
ds_isdir({config},    {ds_init_config_dir({config})})})})


ds_defun({ds_init_config_dir},
{m4_define({ds_global_configdir}, {$1})m4_dnl
AC_CONFIG_AUX_DIR([$1])
AC_SUBST([CONFIGDIR], ["\$(top_srcdir)/$1"])})

#PAGE
## ------------------------------------------------------------
## Finalisation macro.
## ------------------------------------------------------------

ds_defun({ds_output}, {
ds_comment_section({Finalisation.})

AC_CONFIG_FILES([Makefile])
ds_isfile({meta/DESCRIPTION.txt.in},    {AC_CONFIG_FILES([meta/DESCRIPTION.txt])
})m4_dnl

ds_isfile({debian/bin/control.in}, {AC_CONFIG_FILES([debian/bin/control])
})m4_dnl
ds_isfile({debian/dev/control.in}, {AC_CONFIG_FILES([debian/dev/control])
})m4_dnl
ds_isfile({debian/doc/control.in}, {AC_CONFIG_FILES([debian/doc/control])
})m4_dnl
ds_isfile({debian/sta/control.in}, {AC_CONFIG_FILES([debian/sta/control])
})m4_dnl

AC_OUTPUT
})

#PAGE
## ------------------------------------------------------------
## Default macros.
## ------------------------------------------------------------

ds_defun({ds_common_programs}, {ds_comment_section({Common programs.})

AC_PROG_INSTALL
AC_PROG_MAKE_SET

AC_CHECK_PROG([BASH], [bash], [bash], :)
AC_ARG_VAR([BASH], [the GNU bash shell])

AC_CHECK_PROG([RM], [rm], [rm -f -r], :)
AC_ARG_VAR([RM], [program used to delete files and directories recursively])

AC_CHECK_PROG([SYMLINK], [ln], [ln -sv], :)
AC_ARG_VAR([SYMLINK], [program used create symbolic links])

AC_CHECK_PROG([MKDIR], [mkdir], [mkdir -p], :)
AC_ARG_VAR([MKDIR], [program used to create directories recursively])

AC_CHECK_PROG([DPKG], [dpkg], [dpkg], :)
AC_ARG_VAR([DPKG], [program used to build .deb packages])

AC_CHECK_PROG([M4], [m4], [m4], :)
AC_ARG_VAR([M4], [the GNU m4 preprocessor])

AC_PATH_PROG([TCLSH], [tclsh], :)
AC_ARG_VAR([TCLSH], [the TCL shell used to run tests and scripts, full pathname])

AC_CHECK_PROG([MAKEINFO], [makeinfo], [makeinfo], :)
AC_ARG_VAR([MAKEINFO], [program that builds documentation from Texinfo source])

AC_CHECK_PROG([DATE], [date], [date], :)
AC_ARG_VAR([DATE], [the standard date program])

AC_CHECK_PROG([SED], [sed], [sed], :)
AC_CHECK_PROG([GREP], [grep], [grep], :)

CURRENT_DATE=$($DATE)
AC_SUBST(CURRENT_DATE)
})

#PAGE
## ------------------------------------------------------------
## C language macros.
## ------------------------------------------------------------

ds_defun({ds_c_language}, {ds_comment_section({C language environment.})

AC_ARG_ENABLE([assert],
    AC_HELP_STRING([--disable-assert], [Disable assertions.]),
    [AC_DEFINE([NDEBUG])])

AC_PROG_CC
AC_PROG_RANLIB
AC_CHECK_PROG([AR], [ar], [ar], :)
AC_CHECK_PROG([STRIP], [strip], [strip], :)

if test "${{GCC}}" = "yes"; then
    ALL_CFLAGS="${{ALL_CFLAGS}} -Wall -pipe -ansi -pedantic"
fi	
AC_SUBST(ALL_CFLAGS)
AC_ARG_VAR([ALL_CFLAGS], [fixed C compiler flags])

INCLUDES="-I. -I\$(top_srcdir) -I\$(srcdir) -I\$(builddir)"
for ds_tmp in generic compat unix win mac; do
    ds_tmp="${{top_srcdir}}/src/${{ds_tmp}}"
    if test -d "${{ds_tmp}}"; then
	INCLUDES="${{INCLUDES}} -I${{ds_tmp}}"
    fi
done
for ds_tmp in generic compat unix win mac; do
    ds_tmp="${{top_srcdir}}/${{ds_tmp}}"
    if test -d "${{ds_tmp}}"; then
	INCLUDES="${{INCLUDES}} -I${{ds_tmp}}"
    fi
done

AC_SUBST(INCLUDES)
AC_SUBST(LIBS)

AC_HEADER_STDC
AC_C_CONST
AC_TYPE_SIZE_T
AC_FUNC_MALLOC
})

#PAGE
## ------------------------------------------------------------
## TEA macros.
## ------------------------------------------------------------

ds_defun({ds_tcl_package_name},        {m4_define(ds_global_tcl_package_name, {$1})})
ds_defun({ds_tea_require_public_tk},   {m4_define(ds_global_tea_public_tk)})
ds_defun({ds_tea_require_private_tcl}, {m4_define(ds_global_tea_private_tcl)})
ds_defun({ds_tea_require_private_tk},  {m4_define(ds_global_tea_private_tk)})
ds_defun({ds_tea_enable_stub_test},    {m4_define(ds_global_tea_enable_stub_test)})

ds_defun({ds_tea}, {ds_comment_section({TCL Extension Architecture environment.})

m4_pattern_allow([LIBOBJS])
m4{}_include(ds_global_configdir{}/tcl.m4)

AC_CHECK_PROG([GDB], [gdb], [gdb], [:])
AC_ARG_VAR([GDB], [the GNU debugger])

PACKAGE=ds_global_package_name
VERSION=ds_global_package_version
m4_ifdef({ds_global_tcl_package_name}, {
AC_SUBST([TCL_PACKAGE_NAME], [ds_global_tcl_package_name])
AC_DEFINE([TCL_QUOTED_PACKAGE_NAME], ["ds_global_tcl_package_name"])
}, {
AC_SUBST([TCL_PACKAGE_NAME], [ds_global_package_name])
AC_DEFINE([TCL_QUOTED_PACKAGE_NAME], ["ds_global_package_name"])
})m4_dnl
AC_SUBST([TCL_PACKAGE_VERSION], [ds_global_package_version])
AC_DEFINE([TCL_QUOTED_PACKAGE_VERSION], ["ds_global_package_version"])
AC_SUBST([ds_global_package_name{}_LIB_FILE])

ds_isfile({meta/pkgIndex.tcl.in}, {AC_CONFIG_FILES([meta/pkgIndex.tcl])
}, {ds_isfile({tclconfig/pkgIndex.tcl.in}, {AC_CONFIG_FILES([meta/pkgIndex.tcl:tclconfig/pkgIndex.tcl.in])
})})m4_dnl
ds_isfile({tclconfig/libs.tcl.in}, {AC_CONFIG_FILES([libs.tcl:tclconfig/libs.tcl.in])
})m4_dnl

AC_ARG_ENABLE([assert],
    AC_HELP_STRING([--disable-assert], [Disable assertions.]),
    [AC_DEFINE([NDEBUG])])

TEA_INIT
TEA_PATH_TCLCONFIG
TEA_LOAD_TCLCONFIG
TEA_PREFIX
TEA_SETUP_COMPILER
TEA_PUBLIC_TCL_HEADERS
m4_ifdef({ds_global_tea_private_tcl}, {TEA_PRIVATE_TCL_HEADERS})
m4_ifdef({ds_global_tea_public_tk},  {TEA_PUBLIC_TK_HEADERS})
m4_ifdef({ds_global_tea_private_tk},  {TEA_PRIVATE_TK_HEADERS})

if test "$TEA_PLATFORM" = "windows" ; then
    AC_DEFINE_UNQUOTED([BUILD_{}ds_global_package_name])
    CLEANFILES="*.lib *.dll *.exp *.ilk *.pdb vc*.pch"
    EXTRA_SOURCES=
else
    CLEANFILES=
    EXTRA_SOURCES=
fi
AC_SUBST([CLEANFILES])
AC_SUBST([EXTRA_SOURCES])

TEA_ENABLE_THREADS
TEA_ENABLE_SHARED
TEA_CONFIG_CFLAGS
TEA_ENABLE_SYMBOLS

if test "$SHARED_BUILD" = "1" ; then
    CFLAGS='$(CFLAGS_DEFAULT) $(CFLAGS_WARNING) $(SHLIB_CFLAGS)'
else
    CFLAGS='$(CFLAGS_DEFAULT) $(CFLAGS_WARNING)'
fi
AC_SUBST(SHARED_BUILD)

AC_DEFINE(USE_TCL_STUBS)

TEA_MAKE_LIB
TEA_PROG_TCLSH

AC_SUBST([tea_LIB_FILE], ["${}ds_global_package_name{}_LIB_FILE"])

m4_ifdef({ds_global_tea_enable_stub_test}, {
# The stub initialisation file is added to the sources.
EXTRA_SOURCES="ds_global_package_name{}StubInit.c"

# Build the names of the stub library and the test stub library.
eval eval "tea_stub_LIB_FILE=lib{}ds_global_package_name{}stub$UNSHARED_LIB_SUFFIX"
tea_stub_test_LIB_FILE="lib{}ds_global_package_name{}test{}ds_global_package_version{}$SHLIB_SUFFIX"

# The library name has the version number in the format "major.minor.patch";
# if we need to link a program to version "major.minor" we need a symbolic link
# whose name has the shorter format. Build the short name.
eval eval "tea_stub_LIB_FILE_LINK=lib{}ds_global_package_name{}stub${DBGX}.a"


# Build the command used to build the library used to test the stub.
MAKE_STUB_TEST_LIB="\$(SHLIB_LD) -o \$(tea_stub_test_LIB_FILE) \$(tea_stub_test_OBJECTS) -L\$(builddir) -l{}ds_global_package_name{}stub{}ds_global_package_version{}${DBGX} \$(SHLIB_LDFLAGS) \$(SHLIB_LDLIBS)"

# Export symbols. MAKE_STUB_LIB is defined and exported in the
# macros of the "tcl.m4" file.

AC_SUBST([PACKAGE_NAME_UPPERCASE], [ds_global_package_uppercase])
AC_SUBST([tea_stub_LIB_FILE])
AC_SUBST([tea_stub_test_LIB_FILE])
AC_SUBST([MAKE_STUB_TEST_LIB])
})
})


### end of file
# Local Variables:
# mode: sh
# page-delimiter: "^#PAGE"
# End:
m4_divert(0)m4_dnl
