# tests.test --
# 
# Part of: BZTCL
# Contents: tests for the BZTCL library
# Date: Wed May 15, 2002
# 
# Abstract
# 
# 
# 
# Copyright (c) 2002, 2003, 2004 Marco Maggi
# 
# The author  hereby grant permission to use,  copy, modify, distribute,
# and  license this  software  and its  documentation  for any  purpose,
# provided that  existing copyright notices  are retained in  all copies
# and that  this notice  is included verbatim  in any  distributions. No
# written agreement, license, or royalty  fee is required for any of the
# authorized uses.  Modifications to this software may be copyrighted by
# their authors and need not  follow the licensing terms described here,
# provided that the new terms are clearly indicated on the first page of
# each file where they apply.
# 
# IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
# FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
# ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
# DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
# POSSIBILITY OF SUCH DAMAGE.
# 
# THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
# INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
# MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
# NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
# AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
# MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
#

#PAGE
## ------------------------------------------------------------
## Test namespace.
## ------------------------------------------------------------

namespace eval test {

    variable	bzip2	[auto_execok bzip2]
    variable	bunzip2	[auto_execok bunzip2]
    ::tcltest::testConstraint bzip2_available   [expr {[string length $bzip2]>0}]
    ::tcltest::testConstraint bunzip2_available [expr {[string length $bunzip2]>0}]

    
    proc get_this_library { libname } {
	set id [open $libname {RDONLY}]
	fconfigure $id -translation binary -encoding binary
	set data [read $id]
	close $id
	return $data
    }

    variable	data	"abcdefghilmnopqrstuvz"
    variable	data1	[::tcltest::viewFile [info script]]
    variable	data2	[get_this_library $::libfile]

    variable	testdir		[file normalize [::tcltest::configure -testdir]]
    variable	fileName	[file join $testdir proof.bz2]
    variable	otherName	[file join $testdir proof1.bz2]


    # If true [debug] will print a message.
    variable	debugging 0

    proc debug { string } {
	variable	debugging
	if { !$debugging } { return }
	puts stdout [format "%-25s--- %s" test $string]
	flush stdout
    }

    proc compress_file { fileName data } {
	variable	bzlib

	set channel [$bzlib open $fileName WRONLY]
	fconfigure $channel -encoding binary -translation binary
	puts -nonewline $channel $data
	close $channel
    }

    proc decompress_file { fileName {chunkSize 0} } {
	variable	bzlib
	
	set channel [$bzlib open $fileName RDONLY]
	fconfigure $channel -encoding binary -translation binary
	if { $chunkSize > 0 } {
	    while { ! [eof $channel] } {
		append unzipped [read $channel $chunkSize]
	    }
	} else {
	    set unzipped [read $channel]
	}
	close $channel
	return $unzipped
    }

    proc compress_decompress { fileName data {chunkSize 0} } {
	compress_file $fileName $data
	return [decompress_file $fileName $chunkSize]
    }

    proc test_file { fileName data {chunkSize 0} } {
	set result [string equal $data \
			[compress_decompress $fileName $data $chunkSize]]
	::tcltest::removeFile $fileName
	return $result
    }

    proc viewFile { fileName } {
	set channel [open $fileName RDONLY]
	fconfigure $channel -translation binary -encoding binary
	set data [read $channel]
	close $channel
	return $data
    }

    proc makeFile { fileName data } {
	set channel [open $fileName {CREAT EXCL WRONLY}]
	fconfigure $channel -translation binary -encoding binary
	puts -nonewline $channel $data
	close $channel
    }
    
#PAGE

::tcltest::test compress-1.1 "compress a string" {
    -body	{
	catch {$bzlib compress $data}
    }
    -result 0
}

::tcltest::test compress-1.2 "compress and decompress a string" {
    -setup	{
	unset -nocomplain -- size
    }
    -body	{
	set size	[string length $data]
	set zipped	[$bzlib compress $data]
	set unzipped	[$bzlib decompress $zipped -size $size]
	string equal $data $unzipped
    }
    -result 1
    -cleanup	{
	unset -nocomplain unzipped zipped size
    }
}

::tcltest::test compress-1.3 "compress and decompress with options" {
    -setup	{
	unset -nocomplain -- size
    }
    -body	{
	set size	[string length $data]
	set zipped	[$bzlib compress $data 	-workFactor 50 -blockSize 5]
	set unzipped	[$bzlib decompress $zipped -size $size -small]
	string equal $data $unzipped
    }
    -result 1
    -cleanup	{
	unset -nocomplain unzipped zipped size
    }
}

::tcltest::test compress-1.4 "compress and decompress with default options" {
    -body	{
	set size	[string length $data]
	set zipped	[$bzlib compress $data \
		-workFactor default -blockSize default]
	set unzipped	[$bzlib decompress $zipped -size $size -small]
	string equal $data $unzipped
    }
    -result 1
    -cleanup	{
	unset -nocomplain unzipped zipped size
    }
}

::tcltest::test compress-1.5 "compress and decompress a string with no size" {
    -body	{
	set zipped	[$bzlib compress $data]
	set unzipped	[$bzlib decompress $zipped]
	string equal $data $unzipped
    }
    -result 1
    -cleanup	{
	unset -nocomplain unzipped zipped
    }
}

::tcltest::test compress-1.6 "compress a null string" {
    -body	{
	$bzlib compress {}
    }
    -result {}
}

::tcltest::test compress-1.7 "compress a null string, with size" {
    -body	{
	$bzlib compress {}
    }
    -result {}
    -cleanup	{
	unset -nocomplain res
    }
}

::tcltest::test compress-1.8 "decompress a null string" {
    -body	{
	$bzlib decompress {}
    }
    -result {}
}

::tcltest::test compress-1.9 "decompress a null string, with size" {
    -body	{
	$bzlib decompress {} -size 0
    }
    -result {}
}

#PAGE

::tcltest::test compress-2.1 "compress this test file" {
    -body	{
	catch {$bzlib compress $data1}
    }
    -result 0
}

::tcltest::test compress-2.2 "compress and decompress this test file" {
    -body	{
	set size	[string length $data1]
	set zipped	[$bzlib compress $data1]
	set unzipped	[$bzlib decompress $zipped -size $size]
	string equal $data1 $unzipped
    }
    -result 1
    -cleanup	{
	unset -nocomplain unzipped zipped
    }
}

::tcltest::test compress-2.3 "comp/decomp this test file with no size" {
    -body	{
	set zipped	[$bzlib compress $data1]
	set unzipped	[$bzlib decompress $zipped]
	string equal $data1 $unzipped
    }
    -result 1
    -cleanup	{
	unset -nocomplain unzipped zipped
    }
}

#PAGE

::tcltest::test compress-3.1 "compress the bztcl library" {
    -body	{
	catch {$bzlib compress $data2}
    }
    -result 0
}

::tcltest::test compress-3.2 "compress and decompress the bztcl library" {
    -body	{
	set size	[string length $data2]
	set zipped	[$bzlib compress $data2]
	set unzipped	[$bzlib decompress $zipped -size $size]

	expr {
	    ([string length $data2] == $size)
	    && [string equal $data2 $unzipped]
	}
    }
    -result 1
    -cleanup	{
	unset -nocomplain unzipped zipped size
    }
}

::tcltest::test compress-3.3 "comp/decomp this library with no size" {
    -body	{
	set zipped	[$bzlib compress $data2]
	set unzipped	[$bzlib decompress $zipped]
	string equal $data2 $unzipped
    }
    -result 1
    -cleanup	{
	unset -nocomplain unzipped zipped
    }
}

#PAGE

::tcltest::test compress-4.1 "compress and decompress a lot of data" {
    -setup	{
	unset -nocomplain -- size
	set data111 [string repeat a 1500000]
    }
    -body	{
	set size	[string length $data111]
	set zipped	[$bzlib compress $data111]
	set unzipped	[$bzlib decompress $zipped -size $size]

	expr {
	    ([string length $data111] == $size)
	    && [string equal $data111 $unzipped]
	}
    }
    -result 1
    -cleanup	{
	unset -nocomplain unzipped zipped
    }
}

::tcltest::test compress-4.2 "comp/decomp lot of data with no size" {
    -setup	{
	set data111 [string repeat a 1500000]
    }
    -body	{
	set zipped	[$bzlib compress $data111]
	set unzipped	[$bzlib decompress $zipped]
	string equal $data111 $unzipped
    }
    -result 1
    -cleanup	{
	unset -nocomplain unzipped zipped
    }
}

#PAGE

::tcltest::test file-1.1 "comp/decomp string file" {
    -body { test_file $fileName $data }
    -result 1
}

::tcltest::test file-1.2 "comp/decomp string file, chunks" {
    -body { test_file $fileName $data 5 }
    -result 1
}

::tcltest::test file-1.3 "comp/decomp this test file" {
    -body { test_file $fileName $data1 }
    -result 1
}

::tcltest::test file-1.4 "comp/decomp this test file, chunks" {
    -body { test_file $fileName $data1 256 }
    -result 1
}

::tcltest::test file-1.5 "comp/decomp the library" {
    -body { test_file $fileName $data2 }
    -result 1
}

::tcltest::test file-1.5 "comp/decomp the library, chunks" {
    -body { test_file $fileName $data2 4096 }
    -result 1
}

#PAGE

::tcltest::test file-2.1 "comp/decom this library with options" {
    -body	{
	set channel [$bzlib open $fileName WRONLY -blockSize 2 -workFactor 60]
	fconfigure $channel -encoding binary -translation binary
	puts -nonewline $channel $data2
	close $channel

	set channel [$bzlib open $fileName RDONLY -small]
	fconfigure $channel -encoding binary -translation binary
	while { ! [eof $channel] } {
	    append unzipped [read $channel 1024]
	}
	close $channel

	string equal $data2 $unzipped
    }
    -result 1
    -cleanup	{
	::tcltest::removeFile $fileName
	unset -nocomplain channel unzipped
    }
}

::tcltest::test file-2.2 "comp/decom this library with default options" {
    -body	{
	set channel [$bzlib open $fileName WRONLY -blockSize default -workFactor default]
	fconfigure $channel -encoding binary -translation binary
	puts -nonewline $channel $data2
	close $channel

	set channel [$bzlib open $fileName RDONLY -small]
	fconfigure $channel -encoding binary -translation binary
	while { ! [eof $channel] } {
	    append unzipped [read $channel 1024]
	}
	close $channel

	string equal $data2 $unzipped
    }
    -result 1
    -cleanup	{
	::tcltest::removeFile $fileName
	unset -nocomplain channel unzipped
    }
}

#PAGE

::tcltest::test file-3.1 "compress compatibility with bzip" {
    -constraints bunzip2_available
    -body	{
	compress_file $fileName $data2

	set channel [open "|$bunzip2 --stdout $fileName" RDONLY]
	fconfigure $channel -translation binary -encoding binary
	set unzipped [read $channel]
	close $channel

	string equal $data2 $unzipped
    }
    -result 1
    -cleanup	{
	::tcltest::removeFile $fileName
	unset -nocomplain channel unzipped
    }
}

::tcltest::test file-3.2 "uncompress compatibility with bzip" {
    -constraints bzip2_available
    -body	{
	exec $bzip2 --stdout $::libfile >$fileName
	set unzipped [decompress_file $fileName]
	string equal $data2 $unzipped
    }
    -result 1
    -cleanup	{
	::tcltest::removeFile $fileName
	unset -nocomplain unzipped
    }
}

#PAGE

::tcltest::test file-4.1 "fcopy" {
    -setup	{
	::tcltest::removeFile $fileName
	::tcltest::removeFile $otherName
    }
    -body	{
	compress_file $fileName $data2
	
	debug "file length [string length [viewFile $fileName]]"
	
	set channel1 [$bzlib open $fileName RDONLY]
 	fconfigure $channel1 -encoding binary -translation binary
	
	set channel2 [$bzlib open $otherName WRONLY]
 	fconfigure $channel2 -encoding binary -translation binary

	set l [fcopy $channel1 $channel2]
	debug "fcopied $l bytes"

	close $channel1
	close $channel2

	debug "file length [string length [viewFile $fileName]]"
	debug "compressed length [string length [viewFile $otherName]]"
	
	string equal [viewFile $fileName] [viewFile $otherName]
    }
    -result	1
    -cleanup	{
	::tcltest::removeFile $fileName
	::tcltest::removeFile $otherName
	unset -nocomplain channel1 channel2
    }
}

::tcltest::test file-4.2 "fcopy" {
    -constraints bzip2_available
    -setup	{
	::tcltest::removeFile $fileName
	::tcltest::removeFile $otherName
    }
    -body	{
	exec $bzip2 --stdout -3 $::libfile >$fileName
	#                    ^^ must be the same in the bzlib command
	
	debug "file length [string length [viewFile $fileName]]"
	
	set channel1 [$bzlib open $fileName RDONLY]
	fconfigure $channel1 -encoding binary -translation binary
	
	set channel2 [$bzlib open $otherName WRONLY]
	fconfigure $channel2 -encoding binary -translation binary

	set l [fcopy $channel1 $channel2]
	debug "fcopied $l bytes"

	close $channel1
	close $channel2

	debug "file length [string length [viewFile $fileName]]"
	debug "compressed length [string length [viewFile $otherName]]"

	string equal \
	    [::tcltest::viewFile $fileName] \
	    [::tcltest::viewFile $otherName]
    }
    -result	1
    -cleanup	{
	::tcltest::removeFile $fileName
	::tcltest::removeFile $otherName
	unset -nocomplain channel1 channel2
    }
}

#PAGE

::tcltest::test info-1.1 "version of bzlib" {
    -body	{
	regexp -- {^[0-9]+\.[0-9]+\.[0-9]+} [$bzlib info version]
    }
    -result 1
}

#PAGE

::tcltest::test info-2.1 "is work factor" {
    -body	{
	$bzlib info isWorkFactor 30
    }
    -result 1
}

::tcltest::test info-2.2 "is work factor" {
    -body	{
	$bzlib info isWorkFactor default
    }
    -result 1
}

::tcltest::test info-2.3 "is work factor" {
    -body	{
	$bzlib info isWorkFactor 0
    }
    -result 1
}

::tcltest::test info-2.4 "is work factor" {
    -body	{
	$bzlib info isWorkFactor 250
    }
    -result 1
}

::tcltest::test info-2.5 "is work factor" {
    -body	{
	$bzlib info isWorkFactor -1
    }
    -result 0
}

::tcltest::test info-2.6 "is work factor" {
    -body	{
	$bzlib info isWorkFactor 251
    }
    -result 0
}

#PAGE

::tcltest::test info-3.1 "is block size" {
    -body	{
	$bzlib info isBlockSize 3
    }
    -result 1
}

::tcltest::test info-3.2 "is block size" {
    -body	{
	$bzlib info isBlockSize 0
    }
    -result 1
}

::tcltest::test info-3.3 "is block size" {
    -body	{
	$bzlib info isBlockSize default
    }
    -result 1
}

::tcltest::test info-3.4 "is block size" {
    -body	{
	$bzlib info isBlockSize 9
    }
    -result 1
}

::tcltest::test info-3.5 "is block size" {
    -body	{
	$bzlib info isBlockSize 10
    }
    -result 0
}

::tcltest::test info-3.6 "is block size" {
    -body	{
	$bzlib info isBlockSize -1
    }
    -result 0
}

#PAGE

::tcltest::test error-1.1 "command with no arguments" \
    -body	{
	catch { $bzlib } res
	list $res $::errorCode
    } \
    -result [list "wrong # args: should be \"$bzlib subcommand ?arg ...?\"" "LOGIC"]


::tcltest::test error-1.2 "main command, unknown subcommand" {
    -body	{
	catch { $bzlib gasp } res
	list $res $::errorCode
    }
    -result { {bad subcommand "gasp": *} LOGIC }
    -match glob
}

#PAGE

::tcltest::test error-2.1 "compress subcommand, wrong num args" \
    -body	{
	catch { $bzlib compress } res
	list $res $errorCode
    } \
    -result [list "wrong # args: should be \"$bzlib compress data ?options?\"" LOGIC]

::tcltest::test error-2.2 "compress subcommand, wrong option" {
    -body	{
	catch { $bzlib compress $data -gulp } res
	list $res $errorCode
    }
    -result { {bad option "-gulp": *} LOGIC }
    -match	glob
}

::tcltest::test error-2.3 "compress subcommand, -workFactor without argument" {
    -body	{
	catch { $bzlib compress $data -workFactor } res
	list $res $errorCode
    }
    -result { {option "-workFactor" requires argument} LOGIC }
}

::tcltest::test error-2.4 "compress subcommand, -workFactor invalid value" {
    -body	{
	catch { $bzlib compress $data -workFactor gasp } res
	list $res $errorCode
    }
    -result { {expected integer but got "gasp": wrong work factor} LOGIC }
}

::tcltest::test error-2.5 "compress subcommand, -workFactor invalid value" {
    -body	{
	catch { $bzlib compress $data -workFactor 5555 } res
	list $res $errorCode
    }
    -result { {work factor out of range (5555) must be 0-250} LOGIC }
}

::tcltest::test error-2.9 "compress subcommand, -blockSize without argument" {
    -body	{
	catch { $bzlib compress $data -blockSize } res
	list $res $errorCode
    }
    -result { {option "-blockSize" requires argument} LOGIC }
}

::tcltest::test error-2.10 "compress subcommand, -blockSize invalid value" {
    -body	{
	catch { $bzlib compress $data -blockSize gasp } res
	list $res $errorCode
    }
    -result { {expected integer but got "gasp": wrong blocks number} LOGIC }
}

::tcltest::test error-2.11 "compress subcommand, -blockSize invalid value" {
    -body	{
	catch { $bzlib compress $data -blockSize 5555 } res
	list $res $errorCode
    }
    -result { {blocks number out of range (5555) must be 0-9} LOGIC }
}

#PAGE

::tcltest::test error-3.1 "uncompress subcommand, wrong num args" \
    -body	{
	catch { $bzlib decompress } res
	list $res $errorCode
    } \
    -result [list "wrong # args: should be \"$bzlib decompress data ?options?\"" LOGIC]

::tcltest::test error-3.2 "uncompress subcommand, -size without argument" {
    -body	{
	catch { $bzlib decompress $data -size } res
	list $res $errorCode
    }
    -result { {option "-size" requires argument} LOGIC }
}

::tcltest::test error-3.3 "compress subcommand, -size invalid value" {
    -body	{
	catch { $bzlib decompress $data -size gasp } res
	list $res $errorCode
    }
    -result { {expected integer but got "gasp"} LOGIC }
}

::tcltest::test error-3.4 "compress subcommand, wrong option" {
    -body	{
	catch { $bzlib decompress $data -gulp } res
	list $res $errorCode
    }
    -result { {bad option "-gulp": *} LOGIC }
    -match	glob
}

#PAGE

::tcltest::test error-4.1 "open subcommand, wrong num args" \
    -body	{
	catch { $bzlib open } res
	list $res $errorCode
    } \
    -result [list "wrong # args: should be \"$bzlib open fileName mode ?options?\"" LOGIC]

::tcltest::test error-4.2 "open subcommand, wrong mode" {
    -body	{
	catch { $bzlib open $fileName gasp } res
	list $res $errorCode [file isfile $fileName]
    }
    -result { {bad mode "gasp": must be RDONLY or WRONLY} LOGIC 0 }
    -cleanup	{
	::tcltest::removeFile $fileName
    }
}

::tcltest::test error-4.3 "open subcommand, -workFactor without argument" {
    -body	{
	catch { $bzlib open $fileName WRONLY -workFactor } res
	list $res $errorCode [file isfile $fileName]
    }
    -result { {option "-workFactor" requires argument} LOGIC 0 }
    -cleanup	{
	::tcltest::removeFile $fileName
    }
}

::tcltest::test error-4.4 "open subcommand, -workFactor invalid value" {
    -body	{
	catch { $bzlib open $fileName RDONLY -workFactor gasp } res
	list $res $errorCode [file isfile $fileName]
    }
    -result { {expected integer but got "gasp": wrong work factor} LOGIC 0 }
    -cleanup	{
	::tcltest::removeFile $fileName
    }
}

::tcltest::test error-4.5 "open subcommand, -workFactor invalid value" {
    -body	{
	catch { $bzlib open $fileName WRONLY -workFactor 5555 } res
	list $res $errorCode [file isfile $fileName]
    }
    -result { {work factor out of range (5555) must be 0-250} LOGIC 0 }
    -cleanup	{
	::tcltest::removeFile $fileName
    }
}

::tcltest::test error-4.6 "open subcommand, -blockSize without argument" {
    -body	{
	catch { $bzlib open $fileName WRONLY -blockSize } res
	list $res $errorCode [file isfile $fileName]
    }
    -result { {option "-blockSize" requires argument} LOGIC 0 }
    -cleanup	{
	::tcltest::removeFile $fileName
    }
}

::tcltest::test error-4.7 "open subcommand, -blockSize invalid value" {
    -body	{
	catch { $bzlib open $fileName WRONLY -blockSize gasp } res
	list $res $errorCode [file isfile $fileName]
    }
    -result { {expected integer but got "gasp": wrong blocks number} LOGIC 0 }
    -match glob
    -cleanup	{
	::tcltest::removeFile $fileName
    }
}

::tcltest::test error-4.8 "open subcommand, bad option" {
    -body	{
	catch { $bzlib open $fileName WRONLY -gasp } res
	list $res $errorCode [file isfile $fileName]
    }
    -result { {bad option "-gasp": *} LOGIC 0 }
    -match glob
    -cleanup	{
	::tcltest::removeFile $fileName
    }
}

::tcltest::test error-4.9 "open subcommand, bad open mode" {
    -body	{
	catch { $bzlib open $fileName gulping } res
	list $res $errorCode [file isfile $fileName]
    }
    -result { {bad mode "gulping": must be RDONLY or WRONLY} LOGIC 0 }
    -match glob
    -cleanup	{
	::tcltest::removeFile $fileName
    }
}

::tcltest::test error-4.10 "error, no such file" {
    -body	{
	catch { $bzlib open $fileName RDONLY } res
	set res
    }
    -result "no such file or directory"
    -cleanup {
	::tcltest::removeFile $fileName
	unset -nocomplain res
    }
}


#PAGE
## ------------------------------------------------------------
## End of tests.
## ------------------------------------------------------------

::tcltest::cleanupTests
}



### end of file
# Local Variables:
# mode: tcl
# End:
